@@ -18,12 +18,12 @@
 #' A helper function for make_time_window_data and make_time_sequence_data. Takes a dataframe, groups it, and
 #' returns proportion looking and relevant transformations
 #'
-#' @param data
-#' @param groups
-#' @param aoi_col
-#' @return dataframe
+#' @param data       The data
+#' @param groups     The groups
+#' @param aoi_col    Name of AOI column
+#' @return dataframe Transformed dataframe
 .make_proportion_looking_summary <-
-  function(data, groups, aoi_col) {
+  function(data, groups, aoi_col, other_dv_columns) {
     
     .logit_adj <- function(prop) {
       non_zero_ind <- which(prop!=0)
@@ -36,12 +36,16 @@
     }
     
     # Group, summarize Samples
+    
+    summarize_expression <- list(
+      SamplesInAOI = interp( ~ sum(AOI_COL, na.rm = TRUE), AOI_COL = aoi_col),
+      SamplesTotal = interp( ~ sum(!is.na(AOI_COL)), AOI_COL = aoi_col) # ignore all NAs
+      )
+    for (other_dv in other_dv_columns) {
+      summarize_expression[[other_dv]] <- interp( ~ mean(OTHER_DV, na.rm=TRUE), OTHER_DV = as.name(other_dv) )
+    }
     df_grouped <- group_by_(data, .dots = groups)
-    df_summarized <- summarize_(df_grouped,
-                                .dots = list(
-                                  SamplesInAOI = interp( ~ sum(AOI_COL, na.rm = TRUE), AOI_COL = aoi_col),
-                                  SamplesTotal = interp( ~ sum(!is.na(AOI_COL)), AOI_COL = aoi_col) # ignore all NAs
-                                ))
+    df_summarized <- summarize_(df_grouped, .dots = summarize_expression)
     
     # Calculate Proportion, Elog, etc.
     aoi <- as.character(aoi_col)
@@ -80,10 +84,110 @@
   }
 
 
+#' Simulate an eyetrackingR dataset
+#' 
+#' This function creates an eyetrackingR dataset (i.e., already run through make_eyetrackingr_data).
+#' This can be helpful for examining the false-alarm and sensitivity of analysis-techniques via 
+#' simulations.
+#' 
+#' @param num_participants        Number of participants
+#' @param num_items_per_condition Number of trials per-subject per-condition.
+#' @param trial_length            How long is the trial (in ms)?
+#' @param pref                    Their preference between the two AOIs in the "high" condition, 
+#'   where 1 is 100% preference for AOI1, and 0 is 100% preference for AOI2. Default is .5 (equal 
+#'   preference). In the "low" condition, their preference between the two AOIs is equal, so default
+#'   is no effect of condition.
+#' @param pref_window             Vector of length two, specifying start and end of time-window in
+#'   which they expressed the preference specified in \code{pref}. Default is the entire trial
+#' @param ...                     Ignored
+#' @export
+#' @return Prints a list of divergence-times.
+simulate_eyetrackingr_data <- function(num_participants= 16, 
+                                       num_items_per_condition = 6, 
+                                       trial_length = 5000, 
+                                       pref =.50, 
+                                       pref_window = c(1,trial_length),
+                                       ...) {
+  
+  .generate_random_trial <- function(potential_switches_per_trial, pref, pref_onset) {
+    
+    # Randomly generate potential switch times via poisson process:
+    switch_diffs_unnormed <- rexp(potential_switches_per_trial+1)
+    switch_diffs_normed <- round((switch_diffs_unnormed/sum(switch_diffs_unnormed))*trial_len) +1 
+    switch_diffs_normed <- switch_diffs_normed[1:potential_switches_per_trial]
+    switch_times_normed <- cumsum(switch_diffs_normed)
+    
+    # Determine preference/which-aoi based on time in trial:
+    inside_pref_wind  <- (switch_times_normed>pref_onset & switch_times_normed<pref_wind[2])
+    aoi_vec_inside_window  <- rbinom(n = length(switch_times_normed), size = 1, prob = pref)
+    aoi_vec_outside_window <- rbinom(n = length(switch_times_normed), size = 1, prob = .5)
+    which_aoi <- ifelse(inside_pref_wind, aoi_vec_inside_window, aoi_vec_outside_window)
+    
+    # Generate list of looks:
+    out <- unlist(lapply(X = seq_along(switch_diffs_normed), FUN = function(i) rep(which_aoi[i], times = switch_diffs_normed[i])))
+    last_ind <- length(out)
+    out <- out[1:trial_len] # make sure vector is of length trial_len
+    out[is.na(out)] <- out[last_ind] # if it was too short before, pad with last place they were looking
+    as.logical(out)
+  }
+  
+  .random_odds <- function(avg_odds = 1) {
+    rnorm(1, mean = avg_odds, sd = .50)
+  }
+  
+  pref_wind <- pref_window/10 - 1
+  pref_wind_len <- pref_wind[2] - pref_wind[1]
+  trial_len = (trial_length/10) - 1
+  switchiness <- round(trial_len / 60)
+  
+  dat <- data_frame(Participant = rep(1:num_participants, each = num_items_per_condition*trial_len) ) %>%
+    group_by(Participant) %>%
+    mutate(.NumSwitches = rpois(1, lambda = switchiness)+1,
+           .SpeedOffset = rnorm(1, sd = 5),
+           Trial = rep(1:num_items_per_condition, each = n() / num_items_per_condition),
+           Item  = Trial,
+           Condition = ifelse( (Participant%%2)==0, "High", "Low"),
+           ParticipantLogOdds = ifelse(Condition == "High", .random_odds(qlogis(pref)), .random_odds(qlogis(.50)) ))  %>%
+    group_by(Trial) %>%
+    mutate(TrialLogOdds = ifelse(Condition == "High", .random_odds(qlogis(pref)), .random_odds(qlogis(.50)) )) %>%
+    group_by(Participant, Trial) %>%
+    mutate(TimeInTrial = (1:n())*10,
+           RT = rexp(1, rate = abs( (50+.SpeedOffset)^-1 ) )*10,# abs prevents error on 1 in a mil. chance it's negative
+           .PrefOnset = pref_wind[1] + RT/10,
+           .PrefOnset = ifelse(.PrefOnset>pref_wind[2], pref_wind[2], .PrefOnset),
+           AOI1 = .generate_random_trial(unique(.NumSwitches), 
+                                         pref = plogis( (unique(TrialLogOdds)+unique(ParticipantLogOdds))/2 ),
+                                         # each trial pref is avg of that item "pref" w/ that participant pref
+                                         pref_onset = unique(.PrefOnset)),
+                                         # each subject is delayed by a certain amount in when their preference emerges
+                                        
+           AOI2 = !AOI1,
+           Trackloss = as.logical(rbinom(n=n(), size=1, prob = .10)) ) %>%
+    ungroup() %>% select(-.PrefOnset, -.NumSwitches, -.SpeedOffset) %>%
+    mutate(Participant = factor(Participant),
+           Condition = factor(Condition),
+           Trial = factor(Trial),
+           Item  = factor(Item))
+  
+  make_eyetrackingr_data(dat, 
+                         treat_non_aoi_looks_as_missing = TRUE,
+                         item_columns = "Item",
+                         participant_column = "Participant", 
+                         trackloss_column = "Trackloss", 
+                         time_column = "TimeInTrial", 
+                         trial_column = "Trial", 
+                         aoi_columns = c("AOI1", "AOI2"))
+}
+
+
+
+
+
+
 # EyetrackingR-Friendly-Subset ----------------------------------------------------------------------------------
 
 #' EyetrackingR friendly subset
-#' @describeIn subset
+#' @describeIn subset time_sequence_data friendly subset
 #' @param x A dataframe to be subsetted
 #' @param ... further arguments to be passed to or from other methods.
 #' @export
@@ -108,37 +212,37 @@ subset.time_sequence_data <- function(x, ...) {
   return(out)
 }
 
-#' @describeIn subset
+#' @describeIn subset time_window_data friendly subset
 #' @export
 subset.time_window_data <- subset.time_sequence_data
 
-#' @describeIn subset
+#' @describeIn subset bin_analysis friendly subset
 #' @export
 subset.bin_analysis <- subset.time_sequence_data
 
-#' @describeIn subset
+#' @describeIn subset boot_splines friendly subset
 #' @export
 subset.boot_splines_data <- subset.time_sequence_data
 
-#' @describeIn subset
+#' @describeIn subset time_cluster_data friendly subset
 #' @export
 subset.time_cluster_data <- subset.time_sequence_data
 
-#' @describeIn subset
+#' @describeIn subset boot_splines_analysis friendly subset
 #' @export
 subset.boot_splines_analysis <- subset.time_sequence_data
 
-#' @describeIn subset
+#' @describeIn subset onset_data friendly subset
 #' @export
 subset.onset_data <- subset.time_sequence_data
 
-#' @describeIn subset
+#' @describeIn subset eyetrackingR friendly subset
 #' @export
 subset.eyetrackingR <- subset.time_sequence_data
 
 # Friendly Dplyr Verbs: Mutate ----------------------------------------------------------------------------------
 #' EyetrackingR friendly mutate
-#' @describeIn mutate_
+#' @describeIn mutate_ eyetrackingR friendly mutate
 #' @param .data An eyetrackingR dataframe
 #' @param ... name value pairs of expressions
 #' @param .dots Used to work around non-standard evaluation. See vignette("nse") for details.
@@ -164,38 +268,38 @@ mutate_.time_sequence_data <- function(.data, ..., .dots) {
   return(out)
 }
 
-#' @describeIn mutate_
+#' @describeIn mutate_ eyetrackingR friendly mutate
 #' @export
 mutate_.time_window_data <- mutate_.time_sequence_data
 
-#' @describeIn mutate_
+#' @describeIn mutate_ eyetrackingR friendly mutate
 #' @export
 mutate_.bin_analysis <- mutate_.time_sequence_data
 
-#' @describeIn mutate_
+#' @describeIn mutate_ eyetrackingR friendly mutate
 #' @export
 mutate_.boot_splines_data <- mutate_.time_sequence_data
 
-#' @describeIn mutate_
+#' @describeIn mutate_ eyetrackingR friendly mutate
 #' @export
 mutate_.time_cluster_data <- mutate_.time_sequence_data
 
-#' @describeIn mutate_
+#' @describeIn mutate_ eyetrackingR friendly mutate
 #' @export
 mutate_.boot_splines_analysis <- mutate_.time_sequence_data
 
-#' @describeIn mutate_
+#' @describeIn mutate_ eyetrackingR friendly mutate
 #' @export
 mutate_.onset_data <- mutate_.time_sequence_data
 
-#' @describeIn mutate_
+#' @describeIn mutate_ eyetrackingR friendly mutate
 #' @export
 mutate_.eyetrackingR <- mutate_.time_sequence_data
 
 # Friendly Dplyr Verbs: GroupBy ----------------------------------------------------------------------------------
 
 #' EyetrackingR friendly group_by
-#' @describeIn group_by_
+#' @describeIn group_by_ eyetrackingR friendly group_by
 #' @param .data An eyetrackingR dataframe
 #' @param ... variables to group by. Duplicated groups will be silently dropped.
 #' @param .dots Used to work around non-standard evaluation. See vignette("nse") for details.
@@ -223,31 +327,31 @@ group_by_.time_sequence_data <- function(.data, ..., .dots, add= FALSE) {
   return(out)
 }
 
-#' @describeIn group_by_
+#' @describeIn group_by_ eyetrackingR friendly group_by
 #' @export
 group_by_.time_window_data <- group_by_.time_sequence_data
 
-#' @describeIn group_by_
+#' @describeIn group_by_ eyetrackingR friendly group_by
 #' @export
 group_by_.bin_analysis <- group_by_.time_sequence_data
 
-#' @describeIn group_by_
+#' @describeIn group_by_ eyetrackingR friendly group_by
 #' @export
 group_by_.boot_splines_data <- group_by_.time_sequence_data
 
-#' @describeIn group_by_
+#' @describeIn group_by_ eyetrackingR friendly group_by
 #' @export
 group_by_.time_cluster_data <- group_by_.time_sequence_data
 
-#' @describeIn group_by_
+#' @describeIn group_by_ eyetrackingR friendly group_by
 #' @export
 group_by_.boot_splines_analysis <- group_by_.time_sequence_data
 
-#' @describeIn group_by_
+#' @describeIn group_by_ eyetrackingR friendly group_by
 #' @export
 group_by_.onset_data <- group_by_.time_sequence_data
 
-#' @describeIn group_by_
+#' @describeIn group_by_ eyetrackingR friendly group_by
 #' @export
 group_by_.eyetrackingR <- group_by_.time_sequence_data
 
@@ -255,7 +359,7 @@ group_by_.eyetrackingR <- group_by_.time_sequence_data
 # Friendly Dplyr Verbs: Filter ----------------------------------------------------------------------------------
 
 #' EyetrackingR friendly filter
-#' @describeIn filter_
+#' @describeIn filter_ eyetrackingR friendly filter
 #' @param .data An eyetrackingR dataframe
 #' @param ... Logical predicates. Multiple conditions are combined with &.
 #' @param .dots Used to work around non-standard evaluation. See vignette("nse") for details.
@@ -281,31 +385,31 @@ filter_.time_sequence_data <- function(.data, ..., .dots) {
   return(out)
 }
 
-#' @describeIn filter_
+#' @describeIn filter_ eyetrackingR friendly filter
 #' @export
 filter_.time_window_data <- filter_.time_sequence_data
 
-#' @describeIn filter_
+#' @describeIn filter_ eyetrackingR friendly filter
 #' @export
 filter_.bin_analysis <- filter_.time_sequence_data
 
-#' @describeIn filter_
+#' @describeIn filter_ eyetrackingR friendly filter
 #' @export
 filter_.boot_splines_data <- filter_.time_sequence_data
 
-#' @describeIn filter_
+#' @describeIn filter_ eyetrackingR friendly filter
 #' @export
 filter_.time_cluster_data <- filter_.time_sequence_data
 
-#' @describeIn filter_
+#' @describeIn filter_ eyetrackingR friendly filter
 #' @export
 filter_.boot_splines_analysis <- filter_.time_sequence_data
 
-#' @describeIn filter_
+#' @describeIn filter_ eyetrackingR friendly filter
 #' @export
 filter_.onset_data <- filter_.time_sequence_data
 
-#' @describeIn filter_
+#' @describeIn filter_ eyetrackingR friendly filter
 #' @export
 filter_.eyetrackingR <- filter_.time_sequence_data
 
@@ -313,7 +417,7 @@ filter_.eyetrackingR <- filter_.time_sequence_data
 # Friendly Dplyr Verbs: Ungroup ----------------------------------------------------------------------------------
 
 #' EyetrackingR friendly ungroup
-#' @describeIn ungroup
+#' @describeIn ungroup EyetrackingR friendly ungroup
 #' @param x An eyetrackingR dataframe
 #' @export
 #' @return An eyetrackingR dataframe, without dplyr groups
@@ -337,31 +441,31 @@ ungroup.time_sequence_data <- function(x) {
   return(out)
 }
 
-#' @describeIn ungroup
+#' @describeIn ungroup EyetrackingR friendly ungroup
 #' @export
 ungroup.time_window_data <- ungroup.time_sequence_data
 
-#' @describeIn ungroup
+#' @describeIn ungroup EyetrackingR friendly ungroup
 #' @export
 ungroup.bin_analysis <- ungroup.time_sequence_data
 
-#' @describeIn ungroup
+#' @describeIn ungroup EyetrackingR friendly ungroup
 #' @export
 ungroup.boot_splines_data <- ungroup.time_sequence_data
 
-#' @describeIn ungroup
+#' @describeIn ungroup EyetrackingR friendly ungroup
 #' @export
 ungroup.time_cluster_data <- ungroup.time_sequence_data
 
-#' @describeIn ungroup
+#' @describeIn ungroup EyetrackingR friendly ungroup
 #' @export
 ungroup.boot_splines_analysis <- ungroup.time_sequence_data
 
-#' @describeIn ungroup
+#' @describeIn ungroup EyetrackingR friendly ungroup
 #' @export
 ungroup.onset_data <- ungroup.time_sequence_data
 
-#' @describeIn ungroup
+#' @describeIn ungroup EyetrackingR friendly ungroup
 #' @export
 ungroup.eyetrackingR <- ungroup.time_sequence_data
 
@@ -369,7 +473,7 @@ ungroup.eyetrackingR <- ungroup.time_sequence_data
 # Friendly Dplyr Verbs: Left-Join ----------------------------------------------------------------------------------
 
 #' EyetrackingR friendly Left-Join
-#' @describeIn left_join
+#' @describeIn left_join  EyetrackingR friendly Left-Join
 #' @param x Left dataframe
 #' @param y Right dataframe
 #' @param by a character vector of variables to join by
@@ -401,31 +505,31 @@ left_join.time_sequence_data <-
     return(out)
   }
 
-#' @describeIn left_join
+#' @describeIn left_join EyetrackingR friendly Left-Join
 #' @export
 left_join.time_window_data <- left_join.time_sequence_data
 
-#' @describeIn left_join
+#' @describeIn left_join EyetrackingR friendly Left-Join
 #' @export
 left_join.bin_analysis <- left_join.time_sequence_data
 
-#' @describeIn left_join
+#' @describeIn left_join EyetrackingR friendly Left-Join
 #' @export
 left_join.boot_splines_data <- left_join.time_sequence_data
 
-#' @describeIn left_join
+#' @describeIn left_join EyetrackingR friendly Left-Join
 #' @export
 left_join.time_cluster_data <- left_join.time_sequence_data
 
-#' @describeIn left_join
+#' @describeIn left_join EyetrackingR friendly Left-Join
 #' @export
 left_join.boot_splines_analysis <- left_join.time_sequence_data
 
-#' @describeIn left_join
+#' @describeIn left_join EyetrackingR friendly Left-Join
 #' @export
 left_join.onset_data <- left_join.time_sequence_data
 
-#' @describeIn left_join
+#' @describeIn left_join EyetrackingR friendly Left-Join
 #' @export
 left_join.eyetrackingR <- left_join.time_sequence_data
 
